# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand


#' Identify top or low fitting observations based on specified diagnostic metric and filtering method.
#'
#' This function identifies top or low fitting observations based on a specified metric and filtering method.
#'
#' @param list_tmb List of glmmTMB objects.
#' @param metric The metric used for diagnostic (e.g., "AIC", "BIC", "logLik", "deviance", "dispersion").
#' @param filter_method The filtering method to be used (e.g., "mad"). 
#'        Feel free to implement your own filetering method
#' @param keep Whether to keep "top" or "low" fitting observations.
#' @param sort Logical indicating whether to sort the results.
#' @param decreasing Logical indicating whether to sort in decreasing order.
#' @param mad_tolerance Tolerance for MAD-based filtering.
#' @return A character vector of row names corresponding to the top or low fitting observations.
#' @export
#'
#' @examples
#' input_var_list <- init_variable()
#' ## -- simulate RNAseq data 
#' mock_data <- mock_rnaseq(input_var_list, 
#'                       n_genes = 5,
#'                       min_replicates  = 3,
#'                       max_replicates = 3,
#'                       basal_expression = 2,
#'                       sequencing_depth = 1e5)
#' ## -- prepare data & fit a model with mixed effect
#' data2fit = prepareData2fit(countMatrix = mock_data$counts, 
#'                         metadata =  mock_data$metadata)
#'l_tmb <- fitModelParallel(formula = kij ~ myVariable, data = data2fit, 
#'                     group_by = "geneID", family = glmmTMB::nbinom2(link = "log"), 
#'                     n.cores = 1)
#' # Identify top fitting observations based on AIC with MAD filtering
#' identifyTopFit(l_tmb, metric = "AIC", filter_method = "mad", keep = "top", 
#'                sort = TRUE, decreasing = TRUE, mad_tolerance = 3)
#'
#' # Identify low fitting observations based on BIC without sorting
#' identifyTopFit(l_tmb, metric = "BIC", filter_method = "mad", keep = "low", sort = FALSE)
#'
#' # Identify top fitting observations based on log-likelihood with MAD filtering and custom tolerance
#' identifyTopFit(l_tmb, metric = "logLik", filter_method = "mad", keep = "top", mad_tolerance = 2)
identifyTopFit <- function(list_tmb, metric = "AIC", filter_method = "mad", keep = "top", 
                           sort = F, decreasing = T, mad_tolerance = 3){
  
  ## -- verif
  invisible(isValidList_tmb(list_tmb))
  stopifnot(metric %in% c("AIC", "BIC", "logLik", "deviance", "dispersion"))
  
  glance_df <- glance_tmb(list_tmb)
  ## -- MAD method
  if (filter_method == "mad"){
    lft_threshold <- get_mad_left_threshold(glance_df[[metric]], mad_tolerance)
    get_mad_user_message(metric, glance_df[[metric]], lft_threshold, keep)
    index2keep <- if (keep == "top") which(glance_df[[metric]] > lft_threshold) 
                  else which(glance_df[[metric]] < lft_threshold)
    id2keep <- rownames(glance_df)[index2keep]
    glance_df <- glance_df[ id2keep , ]
  } ## -- feel free to implement other filtering methods .. 
  
  ## -- sort
  if (isTRUE(sort)){
    id2keep <- rownames(glance_df)[order(glance_df[[metric]], decreasing = decreasing)]
  }
  return(id2keep)
}

#' Calculate the left threshold for MAD-based filtering.
#'
#' This function calculates the left threshold for MAD-based filtering.
#'
#' @param x Vector of values used for filtering.
#' @param tolerance Tolerance value for MAD-based filtering.
#' @return The left threshold value.
#' @importFrom stats median mad
#' @export
#'
#' @examples
#' # Calculate the left threshold for MAD-based filtering with tolerance 3
#' get_mad_left_threshold(c(100, 200, 300), 3)
#' 
#' # Calculate the left threshold for MAD-based filtering with tolerance 2
#' get_mad_left_threshold(c(50, 75, 100), 2)
get_mad_left_threshold <- function(x, tolerance){
  med <- stats::median(x, na.rm = T)
  mad <- stats::mad(x, center = med, 
                    constant = 1, na.rm = TRUE, 
                    low = FALSE, high = FALSE)
  med - (tolerance * mad)
}

#' Generate user message for MAD filtering.
#'
#' This function generates a user message explaining the MAD filtering process.
#'
#' @param var The name of the metric used for filtering.
#' @param x Vector of values used for filtering.
#' @param left_threshold The left threshold for filtering.
#' @param keep Whether to keep "top" or "low" observations.
#' @return A message explaining the MAD filtering process.
#' @export
#'
#' @examples
#' # Generate user message for MAD filtering with top fitting observations
#' get_mad_user_message("AIC", c(100, 200, 300), 150, "top")
#' 
#' # Generate user message for MAD filtering with low fitting observations
#' get_mad_user_message("BIC", c(50, 75, 100), 75, "low")
get_mad_user_message <- function(var, x , left_threshold, keep) {
  message("Based on the specified metric (", var, ") and the MAD filtering method, the following selection criteria were applied:")
  message("1. The MAD-based threshold for considering outliers was calculated.")
  keep_str <- ifelse(keep == "top", "above", "bellow")
  message(paste("2. Values",  keep_str , "the threshold were identified, threshold:",  left_threshold))
  message("3. Summary of selection:")
  nb_keep <- ifelse(keep == "top", length(x[which(x > left_threshold)]), length(x[which(x < left_threshold)]) )
  sub_msg2 <- paste("values", keep_str,  "the threshold")
  message(paste("-", nb_keep, "out of", length(x), "observations had", sub_msg2, "for the", var, "metric."))
}

